@@ -65,6 +65,7 @@ pub struct Client {
     active: bool,
     handle: ClientHandle,
 
+    pub suspended: bool,
     pub viewport_size: (u16, u16),
     pub scroll: (BufferPositionIndex, BufferPositionIndex),
     pub height: u16,
@@ -78,6 +79,7 @@ impl Client {
     fn dispose(&mut self) {
         self.active = false;
 
+        self.suspended = false;
         self.viewport_size = (0, 0);
         self.scroll = (0, 0);
         self.height = 0;
@@ -154,15 +156,23 @@ impl Client {
             this: &Client,
             editor: &Editor,
         ) -> Option<(BufferPositionIndex, BufferPositionIndex)> {
+            let width = this.viewport_size.0 as BufferPositionIndex;
+            if width == 0 {
+                return None;
+            }
+
             let buffer_view = editor.buffer_views.get(this.buffer_view_handle?)?;
             let buffer = editor.buffers.get(buffer_view.buffer_handle)?.content();
 
             let position = buffer_view.cursors.main_cursor().position;
+
+            // TODO: this is only needed because of undo/redo. need to investigate
+            let position = buffer.saturate_position(position);
+
             let line_index = position.line_index;
             let line = buffer.line_at(line_index as _).as_str();
             let column_index = position.column_byte_index;
 
-            let width = this.viewport_size.0 as BufferPositionIndex;
             let height = this.height as BufferPositionIndex;
             let half_height = height / 2;
 
@@ -282,4 +292,3 @@ impl ClientManager {
         self.clients.iter_mut().filter(|c| c.active)
     }
 }
-
