@@ -1,18 +1,18 @@
 use crate::{
+    buffer_position::BufferPositionIndex,
     buffer_view::CursorMovementKind,
     client::Client,
-    command::{replace_to_between_text_markers, CommandManager},
+    command::CommandManager,
     editor::KeysIterator,
     editor_utils::ReadLinePoll,
     lsp,
     mode::{Mode, ModeContext, ModeKind, ModeOperation, ModeState},
-    register::SEARCH_REGISTER,
+    register::{RETURN_REGISTER, SEARCH_REGISTER},
 };
 
 pub struct State {
     on_client_keys: fn(&mut ModeContext, &mut KeysIterator, ReadLinePoll) -> Option<ModeOperation>,
     continuation: Option<String>,
-    line_var_name: String,
     lsp_client_handle: Option<lsp::ClientHandle>,
 }
 
@@ -21,7 +21,6 @@ impl Default for State {
         Self {
             on_client_keys: |_, _, _| None,
             continuation: None,
-            line_var_name: String::new(),
             lsp_client_handle: None,
         }
     }
@@ -42,6 +41,7 @@ impl ModeState for State {
             &mut ctx.editor.string_pool,
             &ctx.editor.buffered_keys,
             keys,
+            &ctx.editor.registers,
         );
         let func = ctx.editor.mode.read_line_state.on_client_keys;
         func(ctx, keys, poll)
@@ -214,25 +214,25 @@ pub mod filter_cursors {
             pattern: &str,
         ) -> bool {
             if range.from.line_index == range.to.line_index {
-                let line = &buffer.line_at(range.from.line_index).as_str()
-                    [range.from.column_byte_index..range.to.column_byte_index];
+                let line = &buffer.line_at(range.from.line_index as _).as_str()
+                    [range.from.column_byte_index as usize..range.to.column_byte_index as usize];
                 line.contains(pattern)
             } else {
-                let line =
-                    &buffer.line_at(range.from.line_index).as_str()[range.from.column_byte_index..];
+                let line = &buffer.line_at(range.from.line_index as _).as_str()
+                    [range.from.column_byte_index as usize..];
                 if line.contains(pattern) {
                     return true;
                 }
 
                 for line_index in (range.from.line_index + 1)..range.to.line_index {
-                    let line = buffer.line_at(line_index).as_str();
+                    let line = buffer.line_at(line_index as _).as_str();
                     if line.contains(pattern) {
                         return true;
                     }
                 }
 
-                let line =
-                    &buffer.line_at(range.to.line_index).as_str()[..range.to.column_byte_index];
+                let line = &buffer.line_at(range.to.line_index as _).as_str()
+                    [..range.to.column_byte_index as usize];
                 line.contains(pattern)
             }
         }
@@ -295,9 +295,9 @@ pub mod split_cursors {
         ) {
             for (index, s) in line.match_indices(pattern) {
                 let mut anchor = start_position;
-                anchor.column_byte_index += index;
+                anchor.column_byte_index += index as BufferPositionIndex;
                 let mut position = anchor;
-                position.column_byte_index += s.len();
+                position.column_byte_index += s.len() as BufferPositionIndex;
                 cursors.add(Cursor { anchor, position });
             }
         }
@@ -323,9 +323,9 @@ pub mod split_cursors {
             for (i, s) in line.match_indices(pattern) {
                 if index != i {
                     let mut anchor = start_position;
-                    anchor.column_byte_index += index;
+                    anchor.column_byte_index += index as BufferPositionIndex;
                     let mut position = start_position;
-                    position.column_byte_index += i;
+                    position.column_byte_index += i as BufferPositionIndex;
                     cursors.add(Cursor { anchor, position });
                 }
 
@@ -336,11 +336,11 @@ pub mod split_cursors {
                 cursors.add(Cursor {
                     anchor: BufferPosition::line_col(
                         start_position.line_index,
-                        start_position.column_byte_index + index,
+                        start_position.column_byte_index + index as BufferPositionIndex,
                     ),
                     position: BufferPosition::line_col(
                         start_position.line_index,
-                        start_position.column_byte_index + line.len(),
+                        start_position.column_byte_index + line.len() as BufferPositionIndex,
                     ),
                 });
             }
@@ -385,16 +385,16 @@ pub mod split_cursors {
             let new_cursors_start_index = cursors[..].len();
 
             if range.from.line_index == range.to.line_index {
-                let line = &buffer.line_at(range.from.line_index).as_str()
-                    [range.from.column_byte_index..range.to.column_byte_index];
+                let line = &buffer.line_at(range.from.line_index as _).as_str()
+                    [range.from.column_byte_index as usize..range.to.column_byte_index as usize];
                 add_matches(&mut cursors, line, pattern, range.from);
             } else {
-                let line =
-                    &buffer.line_at(range.from.line_index).as_str()[range.from.column_byte_index..];
+                let line = &buffer.line_at(range.from.line_index as _).as_str()
+                    [range.from.column_byte_index as usize..];
                 add_matches(&mut cursors, line, pattern, range.from);
 
                 for line_index in (range.from.line_index + 1)..range.to.line_index {
-                    let line = buffer.line_at(line_index).as_str();
+                    let line = buffer.line_at(line_index as _).as_str();
                     add_matches(
                         &mut cursors,
                         line,
@@ -403,8 +403,8 @@ pub mod split_cursors {
                     );
                 }
 
-                let line =
-                    &buffer.line_at(range.to.line_index).as_str()[..range.to.column_byte_index];
+                let line = &buffer.line_at(range.to.line_index as _).as_str()
+                    [..range.to.column_byte_index as usize];
                 add_matches(
                     &mut cursors,
                     line,
@@ -461,7 +461,7 @@ pub mod goto {
                     let buffer = ctx.editor.buffers.get(buffer_view.buffer_handle)?;
                     let buffer = buffer.content();
 
-                    let mut position = BufferPosition::line_col(line_index, 0);
+                    let mut position = BufferPosition::line_col(line_index as _, 0);
                     position = buffer.saturate_position(position);
                     let word = buffer.word_at(position);
                     if word.kind == WordKind::Whitespace {
@@ -545,7 +545,7 @@ pub mod lsp_rename {
 pub mod custom {
     use super::*;
 
-    pub fn enter_mode(ctx: &mut ModeContext, continuation: &str, line_var_name: &str) {
+    pub fn enter_mode(ctx: &mut ModeContext, continuation: String) {
         fn on_client_keys(
             ctx: &mut ModeContext,
             _: &mut KeysIterator,
@@ -554,14 +554,12 @@ pub mod custom {
             match poll {
                 ReadLinePoll::Pending => None,
                 ReadLinePoll::Submitted => {
-                    let mut continuation =
-                        ctx.editor.mode.read_line_state.continuation.take().unwrap();
-                    replace_to_between_text_markers(
-                        &mut continuation,
-                        &ctx.editor.mode.read_line_state.line_var_name,
-                        ctx.editor.read_line.input(),
-                    );
-                    let operation = CommandManager::eval_commands_then_output(
+                    let continuation = ctx.editor.mode.read_line_state.continuation.take().unwrap();
+
+                    ctx.editor
+                        .registers
+                        .set(RETURN_REGISTER, ctx.editor.read_line.input());
+                    let operation = CommandManager::eval_and_then_output(
                         ctx.editor,
                         ctx.platform,
                         ctx.clients,
@@ -589,11 +587,8 @@ pub mod custom {
 
         let state = &mut ctx.editor.mode.read_line_state;
         state.on_client_keys = on_client_keys;
-        state.continuation = Some(ctx.editor.string_pool.acquire_with(continuation));
-        state.line_var_name.clear();
-        state.line_var_name.push_str(line_var_name);
+        state.continuation = Some(continuation);
 
         Mode::change_to(ctx, ModeKind::ReadLine);
     }
 }
-
