@@ -11,7 +11,7 @@ use std::{
 };
 
 use pepper::{
-    application::AnyError,
+    application::{AnyError, ClientApplication},
     editor_utils::hash_bytes,
     platform::{BufPool, Key, ProcessTag, SharedBuf},
     Args,
@@ -99,17 +99,30 @@ impl RawMode {
             let mut original = std::mem::zeroed();
             libc::tcgetattr(libc::STDIN_FILENO, &mut original);
             let mut new = original.clone();
-            new.c_iflag &= !(libc::BRKINT | libc::ICRNL | libc::INPCK | libc::ISTRIP | libc::IXON);
+            new.c_iflag &= !(libc::IGNBRK
+                | libc::BRKINT
+                | libc::PARMRK
+                | libc::ISTRIP
+                | libc::INLCR
+                | libc::IGNCR
+                | libc::ICRNL
+                | libc::IXON);
             new.c_oflag &= !libc::OPOST;
+            new.c_cflag &= !(libc::CSIZE | libc::PARENB);
             new.c_cflag |= libc::CS8;
             new.c_lflag &= !(libc::ECHO | libc::ICANON | libc::ISIG | libc::IEXTEN);
+            new.c_lflag |= libc::NOFLSH;
             new.c_cc[libc::VMIN] = 0;
-            new.c_cc[libc::VTIME] = 1;
-            libc::tcsetattr(libc::STDIN_FILENO, libc::TCSAFLUSH, &new);
+            new.c_cc[libc::VTIME] = 0;
+            libc::tcsetattr(libc::STDIN_FILENO, libc::TCSANOW, &new);
             original
         };
         Self { original }
     }
+
+    pub fn backspace_code(&self) -> u8 {
+        self.original.c_cc[libc::VERASE]
+    }
 }
 impl Drop for RawMode {
     fn drop(&mut self) {
@@ -228,6 +241,23 @@ impl Drop for Process {
     }
 }
 
+pub fn errno() -> libc::c_int {
+    unsafe { *libc::__errno_location() }
+}
+
+pub fn suspend_process(application: &mut ClientApplication, raw_mode: &mut Option<RawMode>) {
+    application.restore_screen();
+    let was_in_raw_mode = raw_mode.is_some();
+    *raw_mode = None;
+
+    unsafe { libc::raise(libc::SIGTSTP) };
+
+    if was_in_raw_mode {
+        *raw_mode = Some(RawMode::enter());
+    }
+    application.reinit_screen();
+}
+
 pub fn get_terminal_size() -> (usize, usize) {
     let mut size: libc::winsize = unsafe { std::mem::zeroed() };
     let result = unsafe {
@@ -244,10 +274,11 @@ pub fn get_terminal_size() -> (usize, usize) {
     (size.ws_col as _, size.ws_row as _)
 }
 
-pub fn parse_terminal_keys(mut buf: &[u8], keys: &mut Vec<Key>) {
+pub fn parse_terminal_keys(mut buf: &[u8], backspace_code: u8, keys: &mut Vec<Key>) {
     loop {
         let (key, rest) = match buf {
             &[] => break,
+            &[b, ref rest @ ..] if b == backspace_code => (Key::Backspace, rest),
             &[0x1b, b'[', b'5', b'~', ref rest @ ..] => (Key::PageUp, rest),
             &[0x1b, b'[', b'6', b'~', ref rest @ ..] => (Key::PageDown, rest),
             &[0x1b, b'[', b'A', ref rest @ ..] => (Key::Up, rest),
@@ -265,7 +296,7 @@ pub fn parse_terminal_keys(mut buf: &[u8], keys: &mut Vec<Key>) {
             &[0x1b, b'[', b'3', b'~', ref rest @ ..] => (Key::Delete, rest),
             &[0x1b, ref rest @ ..] => (Key::Esc, rest),
             &[0x8, ref rest @ ..] => (Key::Backspace, rest),
-            &[b'\n', ref rest @ ..] => (Key::Enter, rest),
+            &[b'\r', ref rest @ ..] => (Key::Enter, rest),
             &[b'\t', ref rest @ ..] => (Key::Tab, rest),
             &[0x7f, ref rest @ ..] => (Key::Delete, rest),
             &[b @ 0b0..=0b11111, ref rest @ ..] => {
@@ -290,3 +321,4 @@ pub fn parse_terminal_keys(mut buf: &[u8], keys: &mut Vec<Key>) {
         keys.push(key);
     }
 }
+
