@@ -39,13 +39,14 @@ macro_rules! theme_colors {
 
 theme_colors! {
     background,
+    active_line_background,
     highlight,
-    statusbar_active_background,
-    statusbar_inactive_background,
     normal_cursor,
     select_cursor,
     insert_cursor,
     inactive_cursor,
+    statusbar_active_background,
+    statusbar_inactive_background,
 
     token_whitespace,
     token_text,
@@ -66,13 +67,14 @@ impl Default for Theme {
 pub fn gruvbox_theme() -> Theme {
     Theme {
         background: Color::from_u32(0x1d2021),
+        active_line_background: Color::from_u32(0x282828),
         highlight: Color::from_u32(0xfabd2f),
-        statusbar_active_background: Color::from_u32(0x504945),
-        statusbar_inactive_background: Color::from_u32(0x282828),
         normal_cursor: Color::from_u32(0xcc241d),
         insert_cursor: Color::from_u32(0xfabd2f),
         select_cursor: Color::from_u32(0x458588),
         inactive_cursor: Color::from_u32(0x504945),
+        statusbar_active_background: Color::from_u32(0x504945),
+        statusbar_inactive_background: Color::from_u32(0x282828),
 
         token_whitespace: Color::from_u32(0x504945),
         token_text: Color::from_u32(0xebdbb2),
